<?php
namespace Options\ConfigAggregator;

use Options\Db\OptionsTableInterface;
use Options\OptionsProviderInterface;
use Psr\Container\ContainerInterface;
use Zend\ServiceManager\ServiceManager;
use Zend\Stdlib\ArrayUtils;

/**
 * Dodaje postavke iz baze u glavnu konfiguraciju.
 *
 * @package Options\ConfigAggregator
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link http://gitlab.ep.local/strig/options
 */
class DatabaseProcessor
{
    /**
     * Dohvaća postavke iz baze i merđa ih u satu konfiguraciju.
     *
     * @param array $processed
     *
     * @return array
     */
    public function __invoke(array $processed)
    {
        $container = $this->getContainer($processed);
        $config = $container->get('config')['options'] ?? [];

        if (!isset($config['providers']) || empty($config['providers'])) {
            return $processed;
        }

        if (!is_array($config['providers'])) {
            throw new \RuntimeException('Ključ `providers` mora biti array.');
        }

        $table = $this->getTable($container, $config);
        $options = $this->getOptions($table, $config['providers']);

        return ArrayUtils::merge($processed, $options);
    }

    /**
     * Stvara privremeni container.
     *
     * @param array $config
     *
     * @return ServiceManager
     */
    private function getContainer(array $config): ServiceManager
    {
        $container = new ServiceManager($config['dependencies']);
        $container->setService('config', $config);

        return $container;
    }

    /**
     * Vraća postavke definirane u datim providerima.
     *
     * @param OptionsTableInterface $table
     *
     * @param array $providers
     *
     * @return array
     */
    private function getOptions(OptionsTableInterface $table, array $providers): array
    {
        $names = [];
        $defaults = [];

        foreach ($providers as $provider) {
            if (is_string($provider)) {
                if (!class_exists($provider, true)) {
                    throw new \RuntimeException(
                        sprintf('Klasa `%s` ne postoji ili ne može biti učitana.', $provider)
                    );
                }

                /** @var OptionsProviderInterface $provider */
                $provider = new $provider();
            }

            if (!$provider instanceof OptionsProviderInterface) {
                throw new \RuntimeException(
                    sprintf(
                    'Provider mora implementirati OptionsProviderInterface, a dobio sam `%s`.',
                        is_object($provider) ? get_class($provider) : gettype($provider)
                    )
                );
            }

            $names = ArrayUtils::merge($names, $provider->getNames());
            $defaults = ArrayUtils::merge($defaults, $provider->getDefaults());
        }

        $options = $table->getOptions($names);

        return ArrayUtils::merge($defaults, $options);
    }

    /**
     * Vraća tablicu s postavkama.
     *
     * @param ContainerInterface $container
     * @param array $config
     *
     * @return OptionsTableInterface
     */
    private function getTable(ContainerInterface $container, array $config): OptionsTableInterface
    {
        if (!isset($config['table']) || empty($config['table'])) {
            throw new \RuntimeException('Nije postavljen ključ `table`.');
        }

        if (!$container->has($config['table'])) {
            throw new \RuntimeException(
                sprintf('Tablica s imenom `%s` nije registrirana u container.', $config['table'])
            );
        }

        return $container->get($config['table']);
    }
}