<?php
namespace Options;

use Options\Db\OptionsTableInterface;
use Zend\Form\FormInterface;
use Zend\ServiceManager\PluginManagerInterface;
use Zend\Stdlib\ArrayUtils;

/**
 * Upravlja postavkama i potrebnim komponentama.
 *
 * @package Options
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link http://gitlab.ep.local/strig/options
 */
class OptionsManager
{
    /** @var array */
    private $config;
    /** @var PluginManagerInterface */
    private $formManager;
    /** @var string[] */
    private $formNames;
    /** @var FormInterface[] */
    private $forms = [];
    /** @var OptionsTableInterface */
    private $table;
    /** @var OptionsProviderInterface[] */
    private $providers;

    /**
     * Konstruktor.
     *
     * @param array $config
     * @param PluginManagerInterface $formManager
     * @param OptionsTableInterface $table
     */
    public function __construct(
        array $config,
        PluginManagerInterface $formManager,
        OptionsTableInterface $table
    ) {
        $this->config = $config;
        $this->formManager = $formManager;
        $this->table = $table;
    }

    /**
     * Vraća sve obrazce za sve postavke za koje ovaj manager zna.
     *
     * @return FormInterface[]
     */
    public function getAllForms(): array
    {
        foreach ($this->getFormNames() as $option => $form) {
            $this->getForm($option);
        }

        return array_values($this->forms);
    }

    /**
     * Vraća obrazac za postavku s datim imenom.
     *
     * @param string $option
     *
     * @return FormInterface
     */
    public function getForm(string $option): FormInterface
    {
        if (array_key_exists($option, $this->forms)) {
            return $this->forms[$option];
        }

        $names = $this->getFormNames();

        if (!array_key_exists($option, $names)) {
            throw new \RuntimeException(
                sprintf('Ne mogu pronaći obrazac za postavku `%s`.', $option)
            );
        }

        $formName = $names[$option];

        /** @var FormInterface $form */
        $form = $this->formManager->get($formName);
        $form->setData([
            'option_name' => $option,
            'option_value' => $this->config[$option] ?? [],
        ]);

        $this->forms[$option] = $form;

        return $form;
    }

    /**
     * Sprema postavku u bazu.
     *
     * @param string $name
     * @param $value
     */
    public function saveOption(string $name, $value)
    {
        $this->table->setOption($name, $value);
    }

    /**
     * Vraća array u kojem su ključevi imena postavki, a vrijednosti obrasci za te postavke.
     *
     * @return string[]
     */
    private function getFormNames(): array
    {
        if (null === $this->formNames) {
            $this->formNames = [];

            foreach ($this->getProviders() as $provider) {
                $this->formNames = ArrayUtils::merge($this->formNames, $provider->getForms());
            }
        }

        return $this->formNames;
    }

    /**
     * Vraća OptionProvidere zadane u konfiguraciji.
     *
     * @return OptionsProviderInterface[]
     */
    private function getProviders(): array
    {
        if (null === $this->providers) {
            $this->providers = [];

            foreach (($this->config['options']['providers'] ?? []) as $provider) {
                if (is_string($provider)) {
                    $provider = new $provider();
                }

                $this->providers[] = $provider;
            }
        }

        return $this->providers;
    }
}